@@ -36,19 +36,20 @@ import six
 import logging
 import numpy as np
 import matplotlib
-from metpy.units import units
 from fs import open_fs
 from fslib.fs_filepicker import getSaveFileNameAndFilter
 from matplotlib import cbook, figure
 from matplotlib.backends.backend_qt5agg import NavigationToolbar2QT, FigureCanvasQTAgg
 import matplotlib.backend_bases
-from mslib import thermolib
-from mslib.utils import config_loader, FatalUserError
+from PyQt5 import QtCore, QtWidgets, QtGui
+
+from mslib.utils.thermolib import convert_pressure_to_vertical_axis_measure
+from mslib.utils import thermolib, FatalUserError
+from mslib.utils.config import config_loader
+from mslib.utils.units import units
 from mslib.msui import mpl_pathinteractor as mpl_pi
 from mslib.msui import mpl_map
 from mslib.msui.icons import icons
-from PyQt5 import QtCore, QtWidgets, QtGui
-from mslib.utils import convert_pressure_to_vertical_axis_measure
 
 PIL_IMAGE_ORIGIN = "upper"
 LAST_SAVE_DIRECTORY = config_loader(dataset="data_dir")
@@ -80,6 +81,11 @@ class MplCanvas(FigureCanvasQTAgg):
         super(MplCanvas, self).updateGeometry()
 
     def get_default_filename(self):
+        """
+        defines the image file name for storing from a view
+
+        return: default png filename of a view
+        """
         result = self.basename + self.default_filename
         if len(result) > 100:
             result = result[:100]
@@ -168,6 +174,9 @@ save_figure_original = NavigationToolbar2QT.save_figure
 
 
 def save_figure(self, *args):
+    """
+    saves the figure dependent to the filepicker_default
+    """
     picker_type = config_loader(dataset="filepicker_default")
     if picker_type in ["default", "qt"]:
         save_figure_original(self, *args)
@@ -401,7 +410,7 @@ class NavigationToolbar(NavigationToolbar2QT):
                 self.set_message(self.mode)
             else:
                 (lat, lon, alt), _ = self.canvas.waypoints_interactor.get_lat_lon(event)
-                self.set_message(f"lat={lat: <6.2f} lon={lon: <7.2f} altitude={alt/100: <.2f}hPa")
+                self.set_message(f"lat={lat: <6.2f} lon={lon: <7.2f} altitude={alt: <3.0f}hft")
         elif not self.sideview:
             self._update_cursor(event)
 
@@ -434,7 +443,11 @@ class NavigationToolbar(NavigationToolbar2QT):
                 (lat, lon), _ = self.canvas.waypoints_interactor.get_lat_lon(event)
                 y_value = convert_pressure_to_vertical_axis_measure(
                     self.canvas.settings_dict["vertical_axis"], event.ydata)
-                self.set_message(f"{self.mode} lat={lat:6.2f} lon={lon:7.2f} altitude={y_value:.2f}")
+                units = {
+                    "pressure altitude": "km",
+                    "flight level": "hft",
+                    "pressure": "hPa"}[self.canvas.settings_dict["vertical_axis"]]
+                self.set_message(f"{self.mode} lat={lat:6.2f} lon={lon:7.2f} altitude={y_value:.2f}{units}")
 
     def _update_buttons_checked(self):
         super(NavigationToolbar, self)._update_buttons_checked()
@@ -523,6 +536,7 @@ class MplSideViewCanvas(MplCanvas):
         # Main axes instance of mplwidget has zorder 99.
         self.imgax = self.fig.add_axes(
             self.ax.get_position(), frameon=True, xticks=[], yticks=[], label="imgax", zorder=0)
+        self.vertical_lines = []
 
         # Sets the default value of sideview fontsize settings from MSSDefaultConfig.
         self.sideview_size_settings = config_loader(dataset="sideview")
@@ -726,10 +740,12 @@ class MplSideViewCanvas(MplCanvas):
                     self.settings_dict["colour_ceiling"])
 
             # Remove all vertical lines
-            vertical_lines = [line for line in self.ax.lines if
-                              all(x == line.get_path().vertices[0, 0] for x in line.get_path().vertices[:, 0])]
-            for line in vertical_lines:
-                self.ax.lines.remove(line)
+            for line in self.vertical_lines[:]:
+                try:
+                    self.ax.lines.remove(line)
+                except ValueError as e:
+                    logging.debug(f"Vertical line was somehow already removed:\n{e}")
+                self.vertical_lines.remove(line)
 
             # Add vertical lines
             if self.settings_dict["draw_verticals"]:
@@ -739,7 +755,8 @@ class MplSideViewCanvas(MplCanvas):
                     if (ipoint < len(highlight) and
                             np.hypot(lat - highlight[ipoint][0],
                                      lon - highlight[ipoint][1]) < 2E-10):
-                        self.ax.axvline(i, color='k', linewidth=2, linestyle='--', alpha=0.5)
+                        self.vertical_lines.append(
+                            self.ax.axvline(i, color='k', linewidth=2, linestyle='--', alpha=0.5))
                         ipoint += 1
 
         self.draw()
@@ -952,6 +969,7 @@ class MplLinearViewCanvas(MplCanvas):
         # If a waypoints model has been passed, create an interactor on it.
         self.waypoints_interactor = None
         self.waypoints_model = None
+        self.vertical_lines = []
         self.basename = "linearview"
         self.draw()
         if model:
@@ -966,7 +984,7 @@ class MplLinearViewCanvas(MplCanvas):
         If no model had been set before, create a new interactor object on the model
         """
         self.waypoints_model = model
-        pass
+
         if self.waypoints_interactor:
             self.waypoints_interactor.set_waypoints_model(model)
         else:
@@ -1029,13 +1047,21 @@ class MplLinearViewCanvas(MplCanvas):
                                                   lons[::tick_index_step])],
                                     rotation=25, horizontalalignment="right")
 
+            # Remove all vertical lines
+            for line in self.vertical_lines[:]:
+                try:
+                    self.ax.lines.remove(line)
+                except ValueError as e:
+                    logging.debug(f"Vertical line was somehow already removed:\n{e}")
+                self.vertical_lines.remove(line)
+
             ipoint = 0
             highlight = [[wp.lat, wp.lon] for wp in self.waypoints_model.waypoints]
             for i, (lat, lon) in enumerate(zip(lats, lons)):
                 if (ipoint < len(highlight) and
                         np.hypot(lat - highlight[ipoint][0],
                                  lon - highlight[ipoint][1]) < 2E-10):
-                    self.ax.axvline(i, color='k', linewidth=2, linestyle='--', alpha=0.5)
+                    self.vertical_lines.append(self.ax.axvline(i, color='k', linewidth=2, linestyle='--', alpha=0.5))
                     ipoint += 1
             self.draw()
 
@@ -1255,7 +1281,7 @@ class MplTopViewCanvas(MplCanvas):
         self.draw()
         self.repaint()
 
-        # Update in case of a projection change
+        # Update in case of a operationion change
         self.waypoints_interactor.update()
 
         self.pdlg.setValue(10)
